@@ -241,7 +241,7 @@ do_decode_1 (GstVaapiDecoder * decoder, GstVaapiParserFrame * frame)
 
   /* Drop frame if there is no slice data unit in there */
   if (G_UNLIKELY (frame->units->len == 0))
-    return GST_VAAPI_DECODER_STATUS_DROP_FRAME;
+    return (GstVaapiDecoderStatus) GST_VAAPI_DECODER_STATUS_DROP_FRAME;
   return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
 
@@ -816,21 +816,6 @@ gst_vaapi_decoder_get_frame_with_timeout (GstVaapiDecoder * decoder,
   if (!out_frame)
     return GST_VAAPI_DECODER_STATUS_ERROR_NO_DATA;
 
-#if !GST_CHECK_VERSION(1,0,0)
-  if (!GST_VIDEO_CODEC_FRAME_IS_DECODE_ONLY (out_frame)) {
-    const guint flags = GST_VAAPI_SURFACE_PROXY_FLAGS (out_frame->user_data);
-    guint out_flags = 0;
-
-    if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_TFF)
-      out_flags |= GST_VIDEO_CODEC_FRAME_FLAG_TFF;
-    if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_RFF)
-      out_flags |= GST_VIDEO_CODEC_FRAME_FLAG_RFF;
-    if (flags & GST_VAAPI_SURFACE_PROXY_FLAG_ONEFIELD)
-      out_flags |= GST_VIDEO_CODEC_FRAME_FLAG_ONEFIELD;
-    GST_VIDEO_CODEC_FRAME_FLAG_SET (out_frame, out_flags);
-  }
-#endif
-
   *out_frame_ptr = out_frame;
   return GST_VAAPI_DECODER_STATUS_SUCCESS;
 }
@@ -908,8 +893,9 @@ gst_interlace_mode_to_string (GstVideoInterlaceMode mode)
       return "interleaved";
     case GST_VIDEO_INTERLACE_MODE_MIXED:
       return "mixed";
+    default:
+      return "<unknown>";
   }
-  return "<unknown>";
 }
 
 void
