@@ -55,7 +55,6 @@ struct _FrameState
   GstVaapiWindow *window;
   GstVaapiSurface *surface;
   GstVaapiVideoPool *surface_pool;
-  struct wl_buffer *buffer;
   struct wl_callback *callback;
 };
 
@@ -71,7 +70,6 @@ frame_state_new (GstVaapiWindow * window)
   frame->window = window;
   frame->surface = NULL;
   frame->surface_pool = NULL;
-  frame->buffer = NULL;
   frame->callback = NULL;
   return frame;
 }
@@ -89,11 +87,6 @@ frame_state_free (FrameState * frame)
   }
   gst_vaapi_video_pool_replace (&frame->surface_pool, NULL);
 
-  if (frame->buffer) {
-    wl_buffer_destroy (frame->buffer);
-    frame->buffer = NULL;
-  }
-
   if (frame->callback) {
     wl_callback_destroy (frame->callback);
     frame->callback = NULL;
@@ -107,13 +100,17 @@ struct _GstVaapiWindowWaylandPrivate
   struct wl_surface *surface;
   struct wl_region *opaque_region;
   struct wl_event_queue *event_queue;
-  FrameState *frame;
+  FrameState *last_frame;
   GstVideoFormat surface_format;
   GstVaapiVideoPool *surface_pool;
   GstVaapiFilter *filter;
+  GstPoll *poll;
+  GstPollFD pollfd;
   guint is_shown:1;
   guint fullscreen_on_show:1;
   guint use_vpp:1;
+  guint is_cancelled:1;
+  volatile guint num_frames_pending;
 };
 
 /**
@@ -161,17 +158,51 @@ gst_vaapi_window_wayland_sync (GstVaapiWindow * window)
 {
   GstVaapiWindowWaylandPrivate *const priv =
       GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
+  struct wl_display *const wl_display =
+      GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
 
-  if (priv->frame) {
-    struct wl_display *const wl_display =
-        GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
+  if (priv->is_cancelled)
+    return FALSE;
 
-    do {
-      if (wl_display_dispatch_queue (wl_display, priv->event_queue) < 0)
-        return FALSE;
-    } while (priv->frame);
+  if (priv->pollfd.fd < 0) {
+    priv->pollfd.fd = wl_display_get_fd (wl_display);
+    gst_poll_add_fd (priv->poll, &priv->pollfd);
+    gst_poll_fd_ctl_read (priv->poll, &priv->pollfd, TRUE);
+  }
+
+  while (g_atomic_int_get (&priv->num_frames_pending) > 0) {
+    while (wl_display_prepare_read_queue (wl_display, priv->event_queue) < 0) {
+      if (wl_display_dispatch_queue_pending (wl_display, priv->event_queue) < 0)
+        goto error;
+    }
+
+    if (wl_display_flush (wl_display) < 0)
+      goto error;
+
+  again:
+    if (gst_poll_wait (priv->poll, GST_CLOCK_TIME_NONE) < 0) {
+      int saved_errno = errno;
+      if (saved_errno == EAGAIN || saved_errno == EINTR)
+        goto again;
+      if (saved_errno == EBUSY) {       /* closing */
+        wl_display_cancel_read (wl_display);
+        priv->is_cancelled = TRUE;
+        break;
+      }
+      goto error;
+    }
+
+    if (wl_display_read_events (wl_display) < 0)
+      goto error;
+    if (wl_display_dispatch_queue_pending (wl_display, priv->event_queue) < 0)
+      goto error;
   }
   return TRUE;
+
+error:
+  priv->is_cancelled = TRUE;
+  GST_ERROR ("Error on dispatching events: %s", g_strerror (errno));
+  return FALSE;
 }
 
 static void
@@ -260,6 +291,9 @@ gst_vaapi_window_wayland_create (GstVaapiWindow * window,
       &shell_surface_listener, priv);
   wl_shell_surface_set_toplevel (priv->shell_surface);
 
+  priv->poll = gst_poll_new (TRUE);
+  gst_poll_fd_init (&priv->pollfd);
+
   if (priv->fullscreen_on_show)
     gst_vaapi_window_wayland_set_fullscreen (window, TRUE);
 
@@ -276,9 +310,12 @@ gst_vaapi_window_wayland_destroy (GstVaapiWindow * window)
   GstVaapiWindowWaylandPrivate *const priv =
       GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
 
-  if (priv->frame) {
-    frame_state_free (priv->frame);
-    priv->frame = NULL;
+  /* Wait for the last frame to complete redraw */
+  gst_vaapi_window_wayland_sync (window);
+
+  if (priv->last_frame) {
+    frame_state_free (priv->last_frame);
+    priv->last_frame = NULL;
   }
 
   if (priv->shell_surface) {
@@ -298,6 +335,8 @@ gst_vaapi_window_wayland_destroy (GstVaapiWindow * window)
 
   gst_vaapi_filter_replace (&priv->filter, NULL);
   gst_vaapi_video_pool_replace (&priv->surface_pool, NULL);
+
+  gst_poll_free (priv->poll);
 }
 
 static gboolean
@@ -311,6 +350,7 @@ gst_vaapi_window_wayland_resize (GstVaapiWindow * window,
 
   GST_DEBUG ("resize window, new size %ux%u", width, height);
 
+  gst_vaapi_video_pool_replace (&priv->surface_pool, NULL);
   if (priv->opaque_region)
     wl_region_destroy (priv->opaque_region);
   GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
@@ -322,19 +362,29 @@ gst_vaapi_window_wayland_resize (GstVaapiWindow * window,
 }
 
 static void
-frame_redraw_callback (void *data, struct wl_callback *callback, uint32_t time)
+frame_done_callback (void *data, struct wl_callback *callback, uint32_t time)
 {
   FrameState *const frame = data;
   GstVaapiWindowWaylandPrivate *const priv =
       GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (frame->window);
 
+  g_atomic_pointer_compare_and_exchange (&priv->last_frame, frame, NULL);
   frame_state_free (frame);
-  if (priv->frame == frame)
-    priv->frame = NULL;
+  g_atomic_int_dec_and_test (&priv->num_frames_pending);
 }
 
 static const struct wl_callback_listener frame_callback_listener = {
-  frame_redraw_callback
+  frame_done_callback
+};
+
+static void
+frame_release_callback (void *data, struct wl_buffer *wl_buffer)
+{
+  wl_buffer_destroy (wl_buffer);
+}
+
+static const struct wl_buffer_listener frame_buffer_listener = {
+  frame_release_callback
 };
 
 static GstVaapiSurface *
@@ -466,13 +516,21 @@ gst_vaapi_window_wayland_render (GstVaapiWindow * window,
   }
 
   /* Wait for the previous frame to complete redraw */
-  if (!gst_vaapi_window_wayland_sync (window))
+  if (!gst_vaapi_window_wayland_sync (window)) {
+    wl_buffer_destroy (buffer);
     return FALSE;
+  }
+
+  if (priv->is_cancelled) {
+    wl_buffer_destroy (buffer);
+    return TRUE;
+  }
 
   frame = frame_state_new (window);
   if (!frame)
     return FALSE;
-  priv->frame = frame;
+  g_atomic_pointer_set (&priv->last_frame, frame);
+  g_atomic_int_inc (&priv->num_frames_pending);
 
   if (need_vpp && priv->use_vpp) {
     frame->surface = surface;
@@ -490,7 +548,12 @@ gst_vaapi_window_wayland_render (GstVaapiWindow * window,
     priv->opaque_region = NULL;
   }
 
-  frame->buffer = buffer;
+  /* @TODO: It is not OK to use internal event_queue here, since there
+   * may still be a WL_BUFFER_RELEASE event when we destroy the
+   * event_queue in gst_vaapi_window_wayland_destroy (). */
+  wl_proxy_set_queue ((struct wl_proxy *) buffer, priv->event_queue);
+  wl_buffer_add_listener (buffer, &frame_buffer_listener, NULL);
+
   frame->callback = wl_surface_frame (priv->surface);
   wl_callback_add_listener (frame->callback, &frame_callback_listener, frame);
 
@@ -500,6 +563,28 @@ gst_vaapi_window_wayland_render (GstVaapiWindow * window,
   return TRUE;
 }
 
+static gboolean
+gst_vaapi_window_wayland_unblock (GstVaapiWindow * window)
+{
+  GstVaapiWindowWaylandPrivate *const priv =
+      GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
+
+  gst_poll_set_flushing (priv->poll, TRUE);
+
+  return TRUE;
+}
+
+static gboolean
+gst_vaapi_window_wayland_unblock_cancel (GstVaapiWindow * window)
+{
+  GstVaapiWindowWaylandPrivate *const priv =
+      GST_VAAPI_WINDOW_WAYLAND_GET_PRIVATE (window);
+
+  gst_poll_set_flushing (priv->poll, FALSE);
+
+  return TRUE;
+}
+
 static void
 gst_vaapi_window_wayland_class_init (GstVaapiWindowWaylandClass * klass)
 {
@@ -515,6 +600,8 @@ gst_vaapi_window_wayland_class_init (GstVaapiWindowWaylandClass * klass)
   window_class->render = gst_vaapi_window_wayland_render;
   window_class->resize = gst_vaapi_window_wayland_resize;
   window_class->set_fullscreen = gst_vaapi_window_wayland_set_fullscreen;
+  window_class->unblock = gst_vaapi_window_wayland_unblock;
+  window_class->unblock_cancel = gst_vaapi_window_wayland_unblock_cancel;
 }
 
 #define gst_vaapi_window_wayland_finalize \
