@@ -21,15 +21,13 @@
  *  Boston, MA 02110-1301 USA
  */
 
-#include "gst/vaapi/sysdeps.h"
+#include "gstcompat.h"
 #include <gst/vaapi/gstvaapidisplay.h>
 #include <gst/vaapi/gstvaapiencoder_h264.h>
 #include <gst/vaapi/gstvaapiutils_h264.h>
 #include "gstvaapiencode_h264.h"
 #include "gstvaapipluginutil.h"
-#if GST_CHECK_VERSION(1,0,0)
 #include "gstvaapivideomemory.h"
-#endif
 
 #define GST_PLUGIN_NAME "vaapiencode_h264"
 #define GST_PLUGIN_DESC "A VA-API based H.264 video encoder"
@@ -44,20 +42,10 @@ GST_DEBUG_CATEGORY_STATIC (gst_vaapi_h264_encode_debug);
 
 /* *INDENT-OFF* */
 static const char gst_vaapiencode_h264_sink_caps_str[] =
-#if GST_CHECK_VERSION(1,1,0)
   GST_VIDEO_CAPS_MAKE_WITH_FEATURES (GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE,
       "{ ENCODED, NV12, I420, YV12 }") ", "
-#else
-  GST_VAAPI_SURFACE_CAPS ", "
-#endif
   GST_CAPS_INTERLACED_FALSE "; "
-#if GST_CHECK_VERSION(1,0,0)
   GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ", "
-#else
-  "video/x-raw-yuv, "
-  "width  = (int) [ 1, MAX ], "
-  "height = (int) [ 1, MAX ], "
-#endif
   GST_CAPS_INTERLACED_FALSE;
 /* *INDENT-ON* */
 
