@@ -22,7 +22,7 @@
  *  Boston, MA 02110-1301 USA
  */
 
-#include "gst/vaapi/sysdeps.h"
+#include "gstcompat.h"
 #include "gstvaapivideocontext.h"
 #if USE_DRM
 # include <gst/vaapi/gstvaapidisplay_drm.h>
@@ -33,6 +33,9 @@
 #if USE_GLX
 # include <gst/vaapi/gstvaapidisplay_glx.h>
 #endif
+#if USE_EGL
+# include <gst/vaapi/gstvaapidisplay_egl.h>
+#endif
 #if USE_WAYLAND
 # include <gst/vaapi/gstvaapidisplay_wayland.h>
 #endif
@@ -58,36 +61,48 @@ static const char *display_types[] = {
   NULL
 };
 
+typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFunc) (const gchar *);
+typedef GstVaapiDisplay *(*GstVaapiDisplayCreateFromHandleFunc) (gpointer);
+
 typedef struct
 {
   const gchar *type_str;
   GstVaapiDisplayType type;
-  GstVaapiDisplay *(*create_display) (const gchar *);
+  GstVaapiDisplayCreateFunc create_display;
+  GstVaapiDisplayCreateFromHandleFunc create_display_from_handle;
 } DisplayMap;
 
+/* *INDENT-OFF* */
 static const DisplayMap g_display_map[] = {
 #if USE_WAYLAND
   {"wayland",
-        GST_VAAPI_DISPLAY_TYPE_WAYLAND,
-      gst_vaapi_display_wayland_new},
+   GST_VAAPI_DISPLAY_TYPE_WAYLAND,
+   gst_vaapi_display_wayland_new,
+   (GstVaapiDisplayCreateFromHandleFunc)
+   gst_vaapi_display_wayland_new_with_display},
 #endif
 #if USE_GLX
   {"glx",
-        GST_VAAPI_DISPLAY_TYPE_GLX,
-      gst_vaapi_display_glx_new},
+   GST_VAAPI_DISPLAY_TYPE_GLX,
+   gst_vaapi_display_glx_new,
+   (GstVaapiDisplayCreateFromHandleFunc)
+   gst_vaapi_display_glx_new_with_display},
 #endif
 #if USE_X11
   {"x11",
-        GST_VAAPI_DISPLAY_TYPE_X11,
-      gst_vaapi_display_x11_new},
+   GST_VAAPI_DISPLAY_TYPE_X11,
+   gst_vaapi_display_x11_new,
+   (GstVaapiDisplayCreateFromHandleFunc)
+   gst_vaapi_display_x11_new_with_display},
 #endif
 #if USE_DRM
   {"drm",
-        GST_VAAPI_DISPLAY_TYPE_DRM,
-      gst_vaapi_display_drm_new},
+   GST_VAAPI_DISPLAY_TYPE_DRM,
+   gst_vaapi_display_drm_new},
 #endif
   {NULL,}
 };
+/* *INDENT-ON* */
 
 static GstVaapiDisplay *
 gst_vaapi_create_display (GstVaapiDisplayType display_type,
@@ -107,105 +122,161 @@ gst_vaapi_create_display (GstVaapiDisplayType display_type,
   return display;
 }
 
+static GstVaapiDisplay *
+gst_vaapi_create_display_from_handle (GstVaapiDisplayType display_type,
+    gpointer handle)
+{
+  GstVaapiDisplay *display;
+  const DisplayMap *m;
+
+  if (display_type == GST_VAAPI_DISPLAY_TYPE_ANY)
+    return NULL;
+
+  for (m = g_display_map; m->type_str != NULL; m++) {
+    if (m->type == display_type) {
+      display = m->create_display_from_handle ?
+          m->create_display_from_handle (handle) : NULL;
+      return display;
+    }
+  }
+  return NULL;
+}
+
+static GstVaapiDisplay *
+gst_vaapi_create_display_from_gl_context (GstObject * gl_context_object)
+{
+#if USE_GST_GL_HELPERS
+  GstGLContext *const gl_context = GST_GL_CONTEXT (gl_context_object);
+  GstGLDisplay *const gl_display = gst_gl_context_get_display (gl_context);
+  gpointer native_display =
+      GSIZE_TO_POINTER (gst_gl_display_get_handle (gl_display));
+  GstVaapiDisplay *display, *out_display;
+  GstVaapiDisplayType display_type;
+
+  switch (gst_gl_display_get_handle_type (gl_display)) {
+#if USE_X11
+    case GST_GL_DISPLAY_TYPE_X11:
+      display_type = GST_VAAPI_DISPLAY_TYPE_X11;
+      break;
+#endif
+#if USE_WAYLAND
+    case GST_GL_DISPLAY_TYPE_WAYLAND:
+      display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
+      break;
+#endif
+    case GST_GL_DISPLAY_TYPE_ANY:{
+      /* Derive from the active window */
+      GstGLWindow *const gl_window = gst_gl_context_get_window (gl_context);
+      const gchar *const gl_window_type = g_getenv ("GST_GL_WINDOW");
+
+      display_type = GST_VAAPI_DISPLAY_TYPE_ANY;
+      if (!gl_window)
+        break;
+      native_display = GSIZE_TO_POINTER (gst_gl_window_get_display (gl_window));
+
+      if (gl_window_type) {
+#if USE_X11
+        if (!display_type && g_strcmp0 (gl_window_type, "x11") == 0)
+          display_type = GST_VAAPI_DISPLAY_TYPE_X11;
+#endif
+#if USE_WAYLAND
+        if (!display_type && g_strcmp0 (gl_window_type, "wayland") == 0)
+          display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
+#endif
+      } else {
+#if USE_X11
+        if (!display_type && GST_GL_HAVE_WINDOW_X11)
+          display_type = GST_VAAPI_DISPLAY_TYPE_X11;
+#endif
+#if USE_WAYLAND
+        if (!display_type && GST_GL_HAVE_WINDOW_WAYLAND)
+          display_type = GST_VAAPI_DISPLAY_TYPE_WAYLAND;
+#endif
+      }
+      break;
+    }
+    default:
+      display_type = GST_VAAPI_DISPLAY_TYPE_ANY;
+      break;
+  }
+  if (!display_type)
+    return NULL;
+
+  display = gst_vaapi_create_display_from_handle (display_type, native_display);
+  if (!display)
+    return NULL;
+
+  switch (gst_gl_context_get_gl_platform (gl_context)) {
+#if USE_EGL
+    case GST_GL_PLATFORM_EGL:{
+      guint gles_version;
+
+      switch (gst_gl_context_get_gl_api (gl_context)) {
+        case GST_GL_API_GLES1:
+          gles_version = 1;
+          goto create_egl_display;
+        case GST_GL_API_GLES2:
+          gles_version = 2;
+          goto create_egl_display;
+        case GST_GL_API_OPENGL:
+        case GST_GL_API_OPENGL3:
+          gles_version = 0;
+        create_egl_display:
+          out_display = gst_vaapi_display_egl_new (display, gles_version);
+          break;
+        default:
+          out_display = NULL;
+          break;
+      }
+      if (!out_display)
+        return NULL;
+      gst_vaapi_display_egl_set_gl_context (GST_VAAPI_DISPLAY_EGL (out_display),
+          GSIZE_TO_POINTER (gst_gl_context_get_gl_context (gl_context)));
+      break;
+    }
+#endif
+    default:
+      out_display = gst_vaapi_display_ref (display);
+      break;
+  }
+  gst_vaapi_display_unref (display);
+  return out_display;
+#endif
+  GST_ERROR ("unsupported GStreamer version %s", GST_API_VERSION_S);
+  return NULL;
+}
+
 gboolean
-gst_vaapi_ensure_display (gpointer element, GstVaapiDisplayType type)
+gst_vaapi_ensure_display (GstElement * element, GstVaapiDisplayType type)
 {
   GstVaapiPluginBase *const plugin = GST_VAAPI_PLUGIN_BASE (element);
   GstVaapiDisplay *display;
-  GstVideoContext *context;
 
-  g_return_val_if_fail (GST_IS_VIDEO_CONTEXT (element), FALSE);
+  g_return_val_if_fail (GST_IS_ELEMENT (element), FALSE);
 
-  context = GST_VIDEO_CONTEXT (element);
-  g_return_val_if_fail (context != NULL, FALSE);
-
-  gst_vaapi_video_context_prepare (context, display_types);
+  gst_vaapi_video_context_prepare (element, display_types);
 
   /* Neighbour found and it updated the display */
-  if (plugin->display
-      && gst_vaapi_display_type_is_compatible (plugin->display_type, type))
+  if (gst_vaapi_plugin_base_has_display_type (plugin, type))
     return TRUE;
 
   /* If no neighboor, or application not interested, use system default */
-  display = gst_vaapi_create_display (type, plugin->display_name);
+  if (plugin->gl_context)
+    display = gst_vaapi_create_display_from_gl_context (plugin->gl_context);
+  else
+    display = gst_vaapi_create_display (type, plugin->display_name);
   if (!display)
     return FALSE;
 
-  gst_vaapi_video_context_propagate (context, display);
+  gst_vaapi_video_context_propagate (element, display);
   GST_VAAPI_PLUGIN_BASE_DISPLAY_REPLACE (plugin, display);
   gst_vaapi_display_unref (display);
   return TRUE;
 }
 
-void
-gst_vaapi_set_display (const gchar * type,
-    const GValue * value, GstVaapiDisplay ** display_ptr)
-{
-  GstVaapiDisplay *display = NULL;
-
-  if (!strcmp (type, "vaapi-display")) {
-    g_return_if_fail (G_VALUE_HOLDS_POINTER (value));
-    display = gst_vaapi_display_new_with_display (g_value_get_pointer (value));
-  } else if (!strcmp (type, "gst-vaapi-display")) {
-    g_return_if_fail (G_VALUE_HOLDS_POINTER (value));
-    display = gst_vaapi_display_ref (g_value_get_pointer (value));
-  }
-#if USE_DRM
-  else if (!strcmp (type, "drm-device")) {
-    gint device;
-    g_return_if_fail (G_VALUE_HOLDS_INT (value));
-    device = g_value_get_int (value);
-    display = gst_vaapi_display_drm_new_with_device (device);
-  } else if (!strcmp (type, "drm-device-path")) {
-    const gchar *device_path;
-    g_return_if_fail (G_VALUE_HOLDS_STRING (value));
-    device_path = g_value_get_string (value);
-    display = gst_vaapi_display_drm_new (device_path);
-  }
-#endif
-#if USE_X11
-  else if (!strcmp (type, "x11-display-name")) {
-    g_return_if_fail (G_VALUE_HOLDS_STRING (value));
-#if USE_GLX
-    display = gst_vaapi_display_glx_new (g_value_get_string (value));
-#endif
-    if (!display)
-      display = gst_vaapi_display_x11_new (g_value_get_string (value));
-  } else if (!strcmp (type, "x11-display")) {
-    g_return_if_fail (G_VALUE_HOLDS_POINTER (value));
-#if USE_GLX
-    display =
-        gst_vaapi_display_glx_new_with_display (g_value_get_pointer (value));
-#endif
-    if (!display)
-      display =
-          gst_vaapi_display_x11_new_with_display (g_value_get_pointer (value));
-  }
-#endif
-#if USE_WAYLAND
-  else if (!strcmp (type, "wl-display")) {
-    struct wl_display *wl_display;
-    g_return_if_fail (G_VALUE_HOLDS_POINTER (value));
-    wl_display = g_value_get_pointer (value);
-    display = gst_vaapi_display_wayland_new_with_display (wl_display);
-  } else if (!strcmp (type, "wl-display-name")) {
-    const gchar *display_name;
-    g_return_if_fail (G_VALUE_HOLDS_STRING (value));
-    display_name = g_value_get_string (value);
-    display = gst_vaapi_display_wayland_new (display_name);
-  }
-#endif
-
-  if (display) {
-    gst_vaapi_display_replace (display_ptr, display);
-    gst_vaapi_display_unref (display);
-  }
-}
-
 gboolean
 gst_vaapi_reply_to_query (GstQuery * query, GstVaapiDisplay * display)
 {
-#if GST_CHECK_VERSION(1,1,0)
   const gchar *type = NULL;
   GstContext *context;
 
@@ -226,88 +297,6 @@ gst_vaapi_reply_to_query (GstQuery * query, GstVaapiDisplay * display)
   gst_context_unref (context);
 
   return TRUE;
-#else
-  GstVaapiDisplayType display_type;
-  const gchar **types;
-  const gchar *type;
-  gint i;
-  gboolean res = FALSE;
-
-  if (GST_QUERY_TYPE (query) != GST_QUERY_CUSTOM)
-    return FALSE;
-
-  if (!display)
-    return FALSE;
-
-  types = gst_video_context_query_get_supported_types (query);
-
-  if (!types)
-    return FALSE;
-
-  display_type = gst_vaapi_display_get_display_type (display);
-  for (i = 0; types[i] && !res; i++) {
-    type = types[i];
-
-    res = TRUE;
-    if (!strcmp (type, "gst-vaapi-display")) {
-      gst_video_context_query_set_pointer (query, type, display);
-    } else if (!strcmp (type, "vaapi-display")) {
-      VADisplay vadpy = gst_vaapi_display_get_display (display);
-      gst_video_context_query_set_pointer (query, type, vadpy);
-    } else {
-      switch (display_type) {
-#if USE_DRM
-        case GST_VAAPI_DISPLAY_TYPE_DRM:{
-          GstVaapiDisplayDRM *const drm_dpy = GST_VAAPI_DISPLAY_DRM (display);
-          if (!strcmp (type, "drm-device-path"))
-            gst_video_context_query_set_string (query, type,
-                gst_vaapi_display_drm_get_device_path (drm_dpy));
-#if 0
-          /* XXX: gst_video_context_query_set_int() does not exist yet */
-          else if (!strcmp (type, "drm-device"))
-            gst_video_context_query_set_int (query, type,
-                gst_vaapi_display_drm_get_device (drm_dpy));
-#endif
-          else
-            res = FALSE;
-          break;
-        }
-#endif
-#if USE_X11
-        case GST_VAAPI_DISPLAY_TYPE_X11:{
-          GstVaapiDisplayX11 *const xvadpy = GST_VAAPI_DISPLAY_X11 (display);
-          Display *const x11dpy = gst_vaapi_display_x11_get_display (xvadpy);
-          if (!strcmp (type, "x11-display"))
-            gst_video_context_query_set_pointer (query, type, x11dpy);
-          else if (!strcmp (type, "x11-display-name"))
-            gst_video_context_query_set_string (query, type,
-                DisplayString (x11dpy));
-          else
-            res = FALSE;
-          break;
-        }
-#endif
-#if USE_WAYLAND
-        case GST_VAAPI_DISPLAY_TYPE_WAYLAND:{
-          GstVaapiDisplayWayland *const wlvadpy =
-              GST_VAAPI_DISPLAY_WAYLAND (display);
-          struct wl_display *const wldpy =
-              gst_vaapi_display_wayland_get_display (wlvadpy);
-          if (!strcmp (type, "wl-display"))
-            gst_video_context_query_set_pointer (query, type, wldpy);
-          else
-            res = FALSE;
-          break;
-        }
-#endif
-        default:
-          res = FALSE;
-          break;
-      }
-    }
-  }
-  return res;
-#endif /* !GST_CHECK_VERSION(1,1,0) */
 }
 
 gboolean
@@ -341,17 +330,12 @@ gst_vaapi_append_surface_caps (GstCaps * out_caps, GstCaps * in_caps)
 gboolean
 gst_vaapi_apply_composition (GstVaapiSurface * surface, GstBuffer * buffer)
 {
-#if GST_CHECK_VERSION(1,0,0)
   GstVideoOverlayCompositionMeta *const cmeta =
       gst_buffer_get_video_overlay_composition_meta (buffer);
   GstVideoOverlayComposition *composition = NULL;
 
   if (cmeta)
     composition = cmeta->overlay;
-#else
-  GstVideoOverlayComposition *const composition =
-      gst_video_buffer_get_overlay_composition (buffer);
-#endif
   return gst_vaapi_surface_set_subpictures_from_composition (surface,
       composition, TRUE);
 }
@@ -359,7 +343,6 @@ gst_vaapi_apply_composition (GstVaapiSurface * surface, GstBuffer * buffer)
 gboolean
 gst_vaapi_value_set_format (GValue * value, GstVideoFormat format)
 {
-#if GST_CHECK_VERSION(1,0,0)
   const gchar *str;
 
   str = gst_video_format_to_string (format);
@@ -368,16 +351,6 @@ gst_vaapi_value_set_format (GValue * value, GstVideoFormat format)
 
   g_value_init (value, G_TYPE_STRING);
   g_value_set_string (value, str);
-#else
-  guint32 fourcc;
-
-  fourcc = gst_video_format_to_fourcc (format);
-  if (!fourcc)
-    return FALSE;
-
-  g_value_init (value, GST_TYPE_FOURCC);
-  gst_value_set_fourcc (value, fourcc);
-#endif
   return TRUE;
 }
 
@@ -414,7 +387,6 @@ set_video_template_caps (GstCaps * caps)
 GstCaps *
 gst_vaapi_video_format_new_template_caps (GstVideoFormat format)
 {
-#if GST_CHECK_VERSION(1,0,0)
   GstCaps *caps;
 
   g_return_val_if_fail (format != GST_VIDEO_FORMAT_UNKNOWN, NULL);
@@ -427,15 +399,11 @@ gst_vaapi_video_format_new_template_caps (GstVideoFormat format)
       "format", G_TYPE_STRING, gst_video_format_to_string (format), NULL);
   set_video_template_caps (caps);
   return caps;
-#else
-  return gst_video_format_new_template_caps (format);
-#endif
 }
 
 GstCaps *
 gst_vaapi_video_format_new_template_caps_from_list (GArray * formats)
 {
-#if GST_CHECK_VERSION(1,0,0)
   GValue v_formats = G_VALUE_INIT;
   GstCaps *caps;
 
@@ -451,23 +419,6 @@ gst_vaapi_video_format_new_template_caps_from_list (GArray * formats)
   gst_caps_set_value (caps, "format", &v_formats);
   set_video_template_caps (caps);
   g_value_unset (&v_formats);
-#else
-  GstCaps *caps, *tmp_caps;
-  guint i;
-
-  g_return_val_if_fail (formats != NULL, NULL);
-
-  caps = gst_caps_new_empty ();
-  if (!caps)
-    return NULL;
-
-  for (i = 0; i < formats->len; i++) {
-    const GstVideoFormat format = g_array_index (formats, GstVideoFormat, i);
-    tmp_caps = gst_vaapi_video_format_new_template_caps (format);
-    if (tmp_caps)
-      gst_caps_append (caps, tmp_caps);
-  }
-#endif
   return caps;
 }
 
@@ -481,7 +432,6 @@ gst_vaapi_video_format_new_template_caps_with_features (GstVideoFormat format,
   if (!caps)
     return NULL;
 
-#if GST_CHECK_VERSION(1,1,0)
   GstCapsFeatures *const features =
       gst_caps_features_new (features_string, NULL);
   if (!features) {
@@ -489,44 +439,52 @@ gst_vaapi_video_format_new_template_caps_with_features (GstVideoFormat format,
     return NULL;
   }
   gst_caps_set_features (caps, 0, features);
-#endif
   return caps;
 }
 
+static GstCaps *
+new_gl_texture_upload_meta_caps (void)
+{
+  return
+      gst_caps_from_string (GST_VIDEO_CAPS_MAKE_WITH_FEATURES
+      (GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META,
+          "{ RGBA, BGRA }"));
+}
+
 GstVaapiCapsFeature
-gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format)
+gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format,
+    GstVideoFormat * out_format_ptr)
 {
   GstVaapiCapsFeature feature = GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY;
-#if GST_CHECK_VERSION(1,1,0)
   guint i, num_structures;
   GstCaps *caps = NULL;
   GstCaps *gl_texture_upload_caps = NULL;
   GstCaps *sysmem_caps = NULL;
   GstCaps *vaapi_caps = NULL;
   GstCaps *out_caps;
+  GstVideoFormat out_format;
 
-  out_caps = gst_pad_peer_query_caps (pad, NULL);
-  if (!out_caps)
+  out_caps = gst_pad_get_allowed_caps (pad);
+  if (!out_caps) {
+    feature = GST_VAAPI_CAPS_FEATURE_NOT_NEGOTIATED;
     goto cleanup;
+  }
 
-  gl_texture_upload_caps =
-      gst_vaapi_video_format_new_template_caps_with_features
-      (GST_VIDEO_FORMAT_RGBA,
-      GST_CAPS_FEATURE_META_GST_VIDEO_GL_TEXTURE_UPLOAD_META);
+  out_format = format == GST_VIDEO_FORMAT_ENCODED ?
+      GST_VIDEO_FORMAT_I420 : format;
+
+  gl_texture_upload_caps = new_gl_texture_upload_meta_caps ();
   if (!gl_texture_upload_caps)
     goto cleanup;
 
-  if (format == GST_VIDEO_FORMAT_ENCODED)
-    format = GST_VIDEO_FORMAT_I420;
-
   vaapi_caps =
-      gst_vaapi_video_format_new_template_caps_with_features (format,
+      gst_vaapi_video_format_new_template_caps_with_features (out_format,
       GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
   if (!vaapi_caps)
     goto cleanup;
 
   sysmem_caps =
-      gst_vaapi_video_format_new_template_caps_with_features (format,
+      gst_vaapi_video_format_new_template_caps_with_features (out_format,
       GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
   if (!sysmem_caps)
     goto cleanup;
@@ -566,13 +524,37 @@ gst_vaapi_find_preferred_caps_feature (GstPad * pad, GstVideoFormat format)
 #endif
   }
 
+  if (out_format_ptr) {
+    if (feature == GST_VAAPI_CAPS_FEATURE_GL_TEXTURE_UPLOAD_META) {
+      GstStructure *structure;
+      gchar *format_str;
+      out_format = GST_VIDEO_FORMAT_UNKNOWN;
+      do {
+        caps = gst_caps_intersect_full (out_caps, gl_texture_upload_caps,
+            GST_CAPS_INTERSECT_FIRST);
+        if (!caps)
+          break;
+        structure = gst_caps_get_structure (caps, 0);
+        if (!structure)
+          break;
+        if (!gst_structure_get (structure, "format", G_TYPE_STRING,
+                &format_str, NULL))
+          break;
+        out_format = gst_video_format_from_string (format_str);
+        g_free (format_str);
+      } while (0);
+      if (!out_format)
+        goto cleanup;
+    }
+    *out_format_ptr = out_format;
+  }
+
 cleanup:
   gst_caps_replace (&gl_texture_upload_caps, NULL);
   gst_caps_replace (&sysmem_caps, NULL);
   gst_caps_replace (&vaapi_caps, NULL);
   gst_caps_replace (&caps, NULL);
   gst_caps_replace (&out_caps, NULL);
-#endif
   return feature;
 }
 
@@ -582,7 +564,6 @@ gst_vaapi_caps_feature_to_string (GstVaapiCapsFeature feature)
   const gchar *str;
 
   switch (feature) {
-#if GST_CHECK_VERSION(1,1,0)
     case GST_VAAPI_CAPS_FEATURE_SYSTEM_MEMORY:
       str = GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY;
       break;
@@ -592,7 +573,6 @@ gst_vaapi_caps_feature_to_string (GstVaapiCapsFeature feature)
     case GST_VAAPI_CAPS_FEATURE_VAAPI_SURFACE:
       str = GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE;
       break;
-#endif
     default:
       str = NULL;
       break;
@@ -603,7 +583,6 @@ gst_vaapi_caps_feature_to_string (GstVaapiCapsFeature feature)
 gboolean
 gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip)
 {
-#if GST_CHECK_VERSION(1,0,0)
   GstVideoInterlaceMode mode;
   const gchar *mode_str;
 
@@ -625,10 +604,6 @@ gst_caps_set_interlaced (GstCaps * caps, GstVideoInfo * vip)
   }
 
   gst_caps_set_simple (caps, "interlace-mode", G_TYPE_STRING, mode_str, NULL);
-#else
-  gst_caps_set_simple (caps, "interlaced", G_TYPE_BOOLEAN,
-      vip ? GST_VIDEO_INFO_IS_INTERLACED (vip) : FALSE, NULL);
-#endif
   return TRUE;
 }
 
@@ -645,7 +620,6 @@ gst_caps_has_vaapi_surface (GstCaps * caps)
   if (num_structures < 1)
     return FALSE;
 
-#if GST_CHECK_VERSION(1,1,0)
   for (i = 0; i < num_structures && !found_caps; i++) {
     GstCapsFeatures *const features = gst_caps_get_features (caps, i);
 
@@ -658,21 +632,6 @@ gst_caps_has_vaapi_surface (GstCaps * caps)
     found_caps = gst_caps_features_contains (features,
         GST_CAPS_FEATURE_MEMORY_VAAPI_SURFACE);
   }
-#else
-  for (i = 0; i < num_structures && !found_caps; i++) {
-    GstStructure *const structure = gst_caps_get_structure (caps, i);
-    GstCaps *test_caps;
-    GstVideoInfo vi;
-
-    test_caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
-    if (!test_caps)
-      continue;
-
-    found_caps = gst_video_info_from_caps (&vi, test_caps) &&
-        GST_VIDEO_INFO_FORMAT (&vi) == GST_VIDEO_FORMAT_ENCODED;
-    gst_caps_unref (test_caps);
-  }
-#endif
   return found_caps;
 }
 
